/*******************************************************************************
     EFEITO SUPER MQUINA
	 
    (Sequncial de 16 leds vermelhos de alto brilho , com som gravado na flash)
        
     (Verso em C - Use compilador free 'WinAvr - verso 2010')
          
     Microcontrolador: ATMega8
	 
     Autor: Cludio Lrios

     Data: 27/09/2024 
     
 
     Objetivo: Reproduzir efeito visual e sonoro usado no viculo 'Kitt'
     da srie de tv 'super mquina'.	 
     Blog PicSource
     
     Uso didtico

*******************************************************************************/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "sup_maq_sound_4k.c"

//defines
#define sup_maq_sound_4k_size 4500 // tamanho do arquivo em bytes
#define VALOR_DESL ((sup_maq_sound_4k_size /42)<<2) //fator de deslocamento de leds
#define NUM_LEDS  16 //NMERO DE LEDS INSTALADOS
#define Q_LEDS     4 //QUANTIDADE DE LEDS ACESOS (SIMULTANEOS PORM COM PWM DIFERENTES)
#define DESLOC_BIT 8 //DESLOCAMENTO DOS BITS EM RELAO AO BIT0 DA VARIAVEL DE 32 BITS

//variveis globais


uint8_t f_dir,f_g;
uint8_t passo;
uint8_t a,b,c;
uint8_t aux;
uint16_t cntc, cnts;
uint32_t sampleCount;
uint32_t tim1;

//TABELA 1
const uint8_t val_pb [] = 
{ 
0x80,0xc0,0xc0,0xc0,
0x40,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
0x00,
};
//TABELA 2
const uint8_t val_pc [] = 
{ 
0x00,0x00,0x80,0xc0,
0xe0,0xf0,0x78,0x3c,
0x1e,0x0f,0x07,0x03,
0x01,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
0x00,
};
//TABELA 3
const uint8_t val_pd [] = 
{ 
0x00,0x00,0x00,0x00,
0x00,0x00,0x00,0x00,
0x00,0x00,0x80,0xc0,
0xe0,0xf0,0x78,0x3c,
0x1e,0x0f,0x07,0x03,
0x01,
};

//rotina main
int main(){
  DDRB=0XFF;//ports todos como sadas 
  DDRC=0XFF;
  DDRD=0XFF;
  PORTB=0;
  PORTC=0;
  PORTD=0;
  
  //shif dos leds setup
  cntc=0;
  cnts=VALOR_DESL;
  f_g=1;  
  passo=0;
  f_dir=1;//1= esq para dir (led1 para led 16)
  
  //PWM SETUP
  TCCR2= _BV(CS20) | _BV(WGM20) | _BV(COM20) | _BV(COM21);//CLK/1 UP=1 PWM_NORMAL
  sampleCount=0;
  tim1=0; 
  TIMSK=_BV(TOIE2);//liga interrupo do timer 2 overflow
  sei();//liga interrupo geral  
  // 
  
  for(;;)
  { 
    
	//no faz nada, sendo tudo feito na interrupo
    
  }//for(;;){
  return 0;
}//int main()




/**********************************************************************
   ROTINA DE INTERRUPO  
***************************************************************/


ISR(TIMER2_OVF_vect) 

{
   // audio from PROGMEM
  OCR2 = pgm_read_byte(&sound[sampleCount++ >> 2]);
 
  
  if ((sampleCount >> 2) >= sup_maq_sound_4k_size )  //  16k/4=4k
  {
    sampleCount=0;   
  }
  
  if(f_g>0)//permite novo valor de deslocamento dos leds
  {
    PORTD= val_pd[passo];
    PORTC= val_pc[passo];
    PORTB &= 0X3F;
    PORTB |= (val_pb[passo] & 0xc0);
    f_g=0;
  }
  
  if(f_dir>0){//f_dir=1
    cntc++;
	if(cntc>=cnts){
	  
	  passo++;
	  if(passo>20)
	  {
	    f_dir=0;cnts=cnts - VALOR_DESL;
		
	  }
	  else
	  {
	    cnts=cnts + VALOR_DESL;
	  }
	  
	  f_g=1;
	}
    
  }
  else{
    cntc--;
	if(cntc<=cnts)
	{
      cnts=cnts - VALOR_DESL;
	  passo--;
	  if(passo==0xff)
	  {
	    passo=0;
	    cntc=0;
	    cnts=VALOR_DESL; 
	    f_dir=1;	   
	    sampleCount=0;
        tim1=0;		
	    while (tim1<0x2F000)tim1++;//delay para ajuste da velocidade	
           
	  }
	  f_g=1;
    }
  }
  
}

